@@ -1,9 +1,12 @@
 //! A finger client library
-//! See https://datatracker.ietf.org/doc/html/rfc1288
+//!
+//! See [RFC 1288](https://datatracker.ietf.org/doc/html/rfc1288), which this library loosely
+//! implements. The main difference is that it parses requests and responses as UTF-8 instead of
+//! (extended) ASCII.
+//!
 //! Be mature.
 //!
 //! Inspired by Elif Batuman's "The Idiot"
-//! https://datatracker.ietf.org/doc/html/draft-ietf-uri-url-finger-02
 
 use std::io::prelude::*;
 use std::net::{TcpStream, ToSocketAddrs};
@@ -30,32 +33,23 @@ impl From<std::io::Error> for Error {
     }
 }
 
+/// A request object, with configuration parameters
 pub struct Request {
     query: String,
     host: String,
     timeout: Option<u64>, // seconds
-                          // max_response_len: Option<u64>,
+    max_response_len: Option<u64>,
 }
 
-/// From the draft spec:
-/// https://datatracker.ietf.org/doc/html/draft-ietf-uri-url-finger-02
-pub fn from_url(url: &str) -> Result<Request, Error> {
-    if url.starts_with("finger://") {
-        let i = url[9..].find("/").unwrap_or(url.len() - 9);
-        let host = &url[9..9 + i];
-        let query = &url[9 + i..].trim_matches('/');
-        return Ok(finger(query, host));
-    }
-    return Err(Error::InvalidFingerURL);
-}
-
-// TODO better host resolving
-/// rfc1288 2.2 is ignored, queries are UTF-8 strings
-/// Uses sensible defaults for timeout and response len
+/// Build a finger request
+///
+/// rfc1288 2.2 is ignored, queries are UTF-8 strings.
 /// Host can include port. Defaults to 79 if not included
 pub fn finger(query: &str, host: &str) -> Request {
     let mut q = String::new();
     q.push_str(query);
+
+    // Fix query if CRLF was omitted
     if !q.ends_with("\r\n") {
         q.push_str("\r\n")
     }
@@ -63,12 +57,27 @@ pub fn finger(query: &str, host: &str) -> Request {
     Request {
         query: q.to_owned(),
         host: host.to_owned(),
-        timeout: Some(10),
-        // max_response_len: Some(1024 * 10), // 10KB
+        timeout: None,
+        max_response_len: None,
     }
 }
+
 impl Request {
-    /// rfc1288 2.2 is ignored, we will allow UTF-8
+    /// Set timeout. Default is no timeout.
+    pub fn timeout(mut self, timeout: u64) -> Request {
+        self.timeout = Some(timeout);
+        self
+    }
+
+    /// Set max bytes to read before closing connection. Default is no limit.
+    pub fn max_response_len(mut self, max: u64) -> Request {
+        self.max_response_len = Some(max);
+        self
+    }
+
+    /// Send the finger request
+    ///
+    /// rfc1288 2.2 is ignored, we will allow UTF-8. Returns a UTF-8 string
     pub fn send(self) -> Result<String, Error> {
         let dest: String;
         if !self.host.contains(":") {
@@ -92,8 +101,11 @@ impl Request {
         stream.set_read_timeout(Some((start + timeout) - Instant::now()))?;
 
         let mut line: Vec<u8> = vec![];
-        stream.read_to_end(&mut line)?;
-
+        match self.max_response_len {
+            Some(mrl) => stream.take(mrl).read_to_end(&mut line)?,
+            None => stream.read_to_end(&mut line)?,
+        };
+        // Finger responses are small, and always text. Parse as string.
         return Ok(str::from_utf8(&line)?.to_owned());
     }
 }
