@@ -7,9 +7,6 @@ fn main() -> Result<(), fingers::Error> {
         println!("Usage: finger [request]@host or finger://host/request");
         process::exit(1);
     }
-    if args[1].starts_with("finger://") {
-        println!("{}", fingers::from_url(&args[1])?.send()?);
-    }
     let user: &str;
     let host: &str;
     let params: Vec<&str> = args[1].split("@").collect();
